package com.ca.chorus.security.ldap.translator;

import java.sql.Types;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.InitialDirContext;
import javax.naming.ldap.LdapContext;

import org.apache.log4j.Logger;
import org.teiid.language.Call;
import org.teiid.language.Command;
import org.teiid.language.QueryExpression;
import org.teiid.metadata.MetadataFactory;
import org.teiid.metadata.RuntimeMetadata;
import org.teiid.metadata.Table;
import org.teiid.translator.ExecutionContext;
import org.teiid.translator.ProcedureExecution;
import org.teiid.translator.ResultSetExecution;
import org.teiid.translator.Translator;
import org.teiid.translator.TranslatorException;
import org.teiid.translator.TypeFacility;
import org.teiid.translator.UpdateExecution;
import org.teiid.translator.ldap.LDAPExecutionFactory;

@Translator(name = "translator-security-ldap")
public class SecurityLdapTranslator extends LDAPExecutionFactory {
	private static final Logger logger = Logger
			.getLogger(SecurityLdapTranslator.class.getName());

	@Override
	public ResultSetExecution createResultSetExecution(QueryExpression command,
			ExecutionContext executionContext, RuntimeMetadata metadata,
			LdapContext context) throws TranslatorException {
		logger.debug("SecurityLdapTranslator createResultSetExecution");
		return super.createResultSetExecution(command, executionContext,
				metadata, context);
	}

	@Override
	public ProcedureExecution createProcedureExecution(Call command,
			ExecutionContext executionContext, RuntimeMetadata metadata,
			LdapContext connection) throws TranslatorException {
		logger.debug("SecurityLdapTranslator createProcedureExecution");
		return super.createProcedureExecution(command, executionContext,
				metadata, connection);
	}

	@Override
	public UpdateExecution createUpdateExecution(Command command,
			ExecutionContext executionContext, RuntimeMetadata metadata,
			LdapContext context) throws TranslatorException {
		logger.debug("SecurityLdapTranslator createUpdateExecution");
		return super.createUpdateExecution(command, executionContext, metadata,
				context);
	}

	@Override
	public void getMetadata(MetadataFactory metadataFactory, LdapContext conn)
			throws TranslatorException {
		Table table = metadataFactory.addTable("people");
		table.setNameInSource("ou=groups,ou=system?SUBTREE_SCOPE?RESTRICT");

		/*org.teiid.metadata.Column column = new org.teiid.metadata.Column();
		column.setName("UserId");
		Datatype dt = new Datatype();
		dt.setDatatypeID(DataTypeManager.DefaultDataTypes.STRING);
		column.setDatatype(dt);
		column.setNameInSource("uid");
		table.addColumn(column);
		org.teiid.metadata.Column column1 = new org.teiid.metadata.Column();
		column1.setDatatype(dt);
		column1.setNameInSource("cn");
		column1.setName("Name");
		table.addColumn(column1);
*/
		metadataFactory.addColumn("objectclass", TypeFacility.getDataTypeNameFromSQLType(Types.VARCHAR), table);
		metadataFactory.addColumn("ou", TypeFacility.getDataTypeNameFromSQLType(Types.VARCHAR), table);
	}

	 public static void main(String[] args) throws NamingException {

	        Hashtable env = new Hashtable();

	        env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
	        env.put(Context.PROVIDER_URL, "ldap://localhost:10389/ou=groups,ou=system");

	        env.put(Context.SECURITY_PRINCIPAL, "uid=admin,ou=system");
	        env.put(Context.SECURITY_CREDENTIALS, "secret");
	        env.put(Context.SECURITY_AUTHENTICATION, "simple");

	        InitialDirContext ctx = new InitialDirContext(env);
	        LdapContext ldapCtx = (LdapContext) ctx.lookup("ou=groups,ou=system");
	        ldapCtx.search("", "", null);
	        Attributes attrs = ctx.getAttributes("");
	        NamingEnumeration enm = attrs.getAll();
	        while (enm.hasMore()) {
	            System.out.println(enm.next());
	        }
	    }
	}

